#!/bin/bash
#

# build_cython.sh  <product_directory> [tar]

usage()
{
   echo "USAGE: `basename ${0}` <product_directory> <p2711|p2713c|p2713d|p2714b> <prof> [tar]"
}

# -------------------------------------------------------------------
# shared boilerplate
# -------------------------------------------------------------------

get_this_dir() 
{
    ( cd / ; /bin/pwd -P ) >/dev/null 2>&1
    if (( $? == 0 )); then
      pwd_P_arg="-P"
    fi
    reldir=`dirname ${0}`
    thisdir=`cd ${reldir} && /bin/pwd ${pwd_P_arg}`
}

get_ssibuildshims()
{
    # make sure we can use the setup alias
    if [ -z ${UPS_DIR} ]
    then
       echo "ERROR: please setup ups"
       exit 1
    fi
    source `${UPS_DIR}/bin/ups setup ${SETUP_UPS}`
    
    setup ssibuildshims ${ssibuildshims_version} -z ${product_dir}
    if [ -z ${SSIBUILDSHIMS_DIR} ]
    then
       echo "ERROR: failed to setup ssibuildshims ${ssibuildshims_version}"
       exit 1
    fi
}

set_qualifiers()
{
  [[ -n "${basequal}" ]] || ssi_die "Qualifiers mandatory"

  qlist=(`echo ${basequal} | sed -e 's/:/ /g' | tr " " "\n"| sort |tr "\n" " "`)
  for qual in "${qlist[@]}"; do
    case $qual in
      p2711|p2713d|p2714|p2714b)
        pqual=$qual
        ;;
      *)
        ssi_die "Unrecognized qualifier $qual"
    esac
  done

  if [[ -z "$pqual" ]]; then
    ssi_die "A python qualifier is mandatory."
  fi
  case ${extraqual} in
    prof)
      :
      ;;
    debug|opt)
      ssi_die "${extraqual} builds of cython are not supported at this time."
      ;;
    *)
      ssi_die "Unrecognized build qualifier ${extraqual}"
  esac
  basequal=${pqual}${bqual:+:${bqual}}
}

set_compiler_flags()
{
    # We don't compile differently if debug, just use different dependencies.
    cflags="-O3 -g -DNDEBUG -fno-omit-frame-pointer"
}

# -------------------------------------------------------------------
# start processing
# -------------------------------------------------------------------

product_dir=${1}
basequal=${2}
extraqual=${3}
maketar=${4}

if [ -z ${product_dir} ]
then
   echo "ERROR: please specify the local product directory" 1>&2
   usage
   exit 1
fi

# -------------------------------------------------------------------
# package name and version
# -------------------------------------------------------------------

package=cython
origpkgver=v0_28_2
pkgver=${origpkgver}
ssibuildshims_version=v1_04_04
pkgdotver=`echo ${origpkgver} | sed -e 's/_/./g' -e 's/^v//'`
pkgtarname=${package}-${pkgdotver}

get_this_dir
get_ssibuildshims

set --
source ssi_functions
set_qualifiers

source define_basics --

set_compiler_flags

if [ "${maketar}" = "tar" ] && [ -d ${pkgdir}/lib ]
then
   ${SSIBUILDSHIMS_DIR}/bin/make_distribution_tarball ${product_dir} ${package} ${pkgver} ${fullqual}
   exit 0
fi

echo "building ${package} for ${OS}-${plat}-${qualdir} (flavor ${flvr})"

# -------------------------------------------------------------------
# package dependent stuff goes here
# -------------------------------------------------------------------

mkdir -p ${blddir} || ssi_die "failed to create ${blddir}"

# declare now so we can setup
# fake ups declare
fakedb=${product_dir}/${package}/${pkgver}/fakedb
${SSIBUILDSHIMS_DIR}/bin/fake_declare_product ${product_dir} ${package} ${pkgver} ${fullqual}  || \
  ssi_die "Fake declare of $package failed"

setup -B ${package} ${pkgver} -q ${fullqual} -z ${fakedb}:${product_dir}:${PRODUCTS} || ssi_die "fake setup failed"

(( pv = ${pqual//[A-Za-z]/} ))
if (( ${pv} < 2713 )); then # Bundled with newer python.
  setup -B setuptools ${setuptools_version} -q ${pqual} -z ${product_dir}:${PRODUCTS} || \
    ssi_die "unable to set up UPS package for python setuptools ${setuptools_version}"
fi

ups active

if [ ${OS1} = "Darwin" ]
then
   PATH=`dropit /usr/bin/get-cert $PATH`
fi

set -x

# work in a build directory, which will not be part of any distribution
python_shortdotversion=${PYTHON_VERSION#v}
python_shortdotversion=${python_shortdotversion//_/.}
python_shortdotversion=${python_shortdotversion%.*}

mkdir -p ${pkgdir}/lib/python${python_shortdotversion}/site-packages || ssi_die "failed to create ${pkgdir}/lib/python${python_shortdotversion}/site-packages"
cd ${blddir} || ssi_die "Unable to cd into ${blddir}"
tar xf ${tardir}/Cython-${pkgdotver}.tar.gz || ssi_die "Failed to untar ${tardir}/Cython-${pkgdotver}.tar.gz"
# The tarball is Cython...
cd Cython-${pkgdotver} || ssi_die "Failed to cd into Cython-${pkgdotver}"
python setup.py install --prefix ${pkgdir} --single-version-externally-managed --root "/" || ssi_die "python setup.py install failed"

set +x

if [ ! -d ${pkgdir}/lib ]
then
   echo "ERROR: failed to create ${pkgdir}/lib"
   echo "       Something is very wrong"
   exit 1
fi

#replace hardwired python path 
sed -i'' -e '1s/^.*$/#!\/usr\/bin\/env python /' ${pkgdir}/bin/cygdb || ssi_die "Failed to fix shebang for cygdb"
sed -i'' -e '1s/^.*$/#!\/usr\/bin\/env python /' ${pkgdir}/bin/cython || ssi_die "Failed to fix shebang for cython"
sed -i'' -e '1s/^.*$/#!\/usr\/bin\/env python /' ${pkgdir}/bin/cythonize || ssi_die "Failed to fix shebang for cythonize"

# real ups declare
${SSIBUILDSHIMS_DIR}/bin/declare_product ${product_dir} ${package} ${pkgver} ${fullqual} || ssi_die "failed to declare ${package} ${pkgver}"

# -------------------------------------------------------------------
# common bottom stuff
# -------------------------------------------------------------------

# this should not complain
echo "Finished building ${package} ${pkgver}"
setup -B ${package} ${pkgver} -q ${fullqual} -z ${product_dir}:${PRODUCTS}
echo "cython is installed at ${CYTHON_FQ_DIR}"

# this must be last
if [ "${maketar}" = "tar" ] && [ -d ${pkgdir}/lib ]
then
   ${SSIBUILDSHIMS_DIR}/bin/make_distribution_tarball ${product_dir} ${package} ${pkgver} ${fullqual}
fi

exit 0
